/* ***** BEGIN LICENSE BLOCK *****
 * 
 * Pearltrees add-on AMO, Copyright(C), 2009, Broceliand SAS, Paris, France 
 * (company in charge of developing Pearltrees)
 * 
 * This file is part of “Pearltrees add-on AMO”.  
 * 
 * Pearltrees add-on AMO is free software: you can redistribute it and/or modify it under the 
 * terms of the GNU General Public License version 3 as published by the Free Software Foundation.
 * 
 * Pearltrees add-on AMO is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with Pearltrees add-on AMO. 
 * If not, see <http://www.gnu.org/licenses/>
 * 
 * ***** END LICENSE BLOCK ***** */

/////////////////////////////////////////////////////////////////////////////////
// Firefox listeners
// Each time a new window is created, we add our listeners.
// So we can register anything the user see on his screen.
//
// Classes:
//
//    BRO_browserManager   Managing the differents browsers
//    BRO_navListener      Listen window location changes and request states
//    BRO_actionListener   Listen to various extensions and components
//    BRO_listenerHandler  Manage all types of listeners
//
/////////////////////////////////////////////////////////////////////////////////

const BRO_METHOD_UNKNOWN = 0;
const BRO_METHOD_TAB_SELECTED = 1;
const BRO_METHOD_LINK_CLICKED = 2;
const BRO_METHOD_FORM_SUBMITED = 3;
const BRO_METHOD_BACK_BUTTON = 4;
const BRO_METHOD_FORWARD_BUTTON = 6;
const BRO_METHOD_TAB_CREATED = 5;
const BRO_METHOD_NEW_WINDOW = 7;
// Never sent to the server
const BRO_METHOD_BODY_CLICKED = 8;    // It is used for identifying javascript buttons.
const BRO_METHOD_IGNORE = 9;          // Never sent to the server.
const BRO_METHOD_CLICK_HOME = 10; 
const BRO_METHOD_SEARCH = 11; 
const BRO_METHOD_TYPE_URL = 12;
const BRO_METHOD_CLICK_BOOKMARK = 12; 

/**
 * Time for firefox to create a new tab
 * @type {Number} ms
 */
const BRO_TIME_TO_CREATE_TAB = 1000; 

/**
 * Time for a page to redirect itself.
 * The longest case found is netvibes which reload the page depending the last netvibes tab selected.
 * It may takes 4000 ms.
 * @type {Number}
 */
const BRO_TIME_TO_REDIRECT = 5000; 

/**
 * This time is used for detecting javascript buttons.
 * Facebook log in around 7000 ms. Most of the tests are around 400 ms.
 * @type {Number}
 */
const BRO_JS_TIME_TO_CHANGE_LOCATION = 10000; // ms = 10sec

/**
 * This class is used for managing the differents browsers.
 * Browsers may be tabs or windows.
 * Basically we asign a unique ID to each new tab. Then we send this ID with each page
 * sent to the server.
 */
var BRO_browserManager = {
	
	init: function() {
		window.addEventListener("load", this.load, false);
	},
    
	/**
	 * When firefox is started we asign IDs to the default browsers & tabs.
	 */
	load:function() {	   
        for (var i = 0; i < gBrowser.browsers.length; i++) {
            var browser = gBrowser.getBrowserAtIndex(i);
            try {
            	BRO_browserManager.initBrowser(browser);
            } catch(e) {
                Components.utils.reportError(e);
            }
        }
	},
	
	/**
	 * Add a BRO_UID param to the browser object.
	 * This UID is generated with the local time and a random number.
	 * @param browser http://developer.mozilla.org/en/docs/XUL:browser
	 * @param parent browser
	 */
	initBrowser:function(browser, parent) {
		var date = new Date();
		var time = date.getTime()+'';
		browser.BRO_UID = time+BRO_tools.randomID(1);
		if(parent) {
			browser.BRO_PARENT_UID = parent.BRO_UID;
		}
	},
	
	getBrowserByID:function(browserID) {
        for (var i = 0; i < gBrowser.browsers.length; i++) {
            var browser = gBrowser.getBrowserAtIndex(i);
            if(browser.BRO_UID == browserID) {
            	return browser;
            }
        }
        return null;
	},
	
	/**
	 * Return the selected browser object
	 * @return browser http://developer.mozilla.org/en/docs/XUL:browser
	 */
	getSelectedBrowser:function() {
        return (gBrowser)?gBrowser.selectedBrowser:null;
	},
	
	/**
	 * Getter for the selected browser unique ID.
	 * @return int Browser ID
	 */
	getSelectedBrowserID:function() {
		if(!this.getSelectedBrowser()) return null;
		else if(!this.getSelectedBrowser().BRO_UID) this.initBrowser(this.getSelectedBrowser());
		return this.getSelectedBrowser().BRO_UID;
	},
	
	/**
	 * Getter for the URL
	 * @return string URL
	 */
	getSelectedBrowserUrl:function() {
		var selectedBrowser = this.getSelectedBrowser();
		return (selectedBrowser && selectedBrowser.currentURI)?selectedBrowser.currentURI.spec:null;
	}
}

//const BRO_NOTIFY_ALL = Components.interfaces.nsIWebProgress.NOTIFY_ALL;
//const BRO_NOTIFY_STATE_ALL = Components.interfaces.nsIWebProgress.NOTIFY_STATE_ALL;
const BRO_NOTIFY_STATE_DOCUMENT = Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT;
const BRO_PROGRESS_LISTENER = Components.interfaces.nsIWebProgressListener;
const BRO_REQUEST = Components.interfaces.nsIRequest;

const BRO_STATE_START = BRO_PROGRESS_LISTENER.STATE_START;
const BRO_STATE_STOP = BRO_PROGRESS_LISTENER.STATE_STOP;
const BRO_STATE_REDIRECTING = BRO_PROGRESS_LISTENER.STATE_REDIRECTING;
const BRO_STATE_TRANSFERRING = BRO_PROGRESS_LISTENER.STATE_TRANSFERRING;
const BRO_STATE_IS_DOCUMENT = BRO_PROGRESS_LISTENER.STATE_IS_DOCUMENT;
const LOAD_FROM_CACHE = BRO_REQUEST.LOAD_FROM_CACHE;

/**
 * Navigation listener. 
 * 
 * We listen to the window location and the request states.
 * The location event is fired when the URL changes.
 * The progress event is fired when requests are sent / received.
 * 
 * We notify the server with the page description when enaugh content is loaded.
 * This may be happen immediately when the page is already in the cache (this is the
 * case when switching beetwen tabs) or when the HTTP response is received. 
 * Currently we wait for a valid URL and a valid title.
 * 
 * @type nsIWebProgressListener
 */
var BRO_navListener = {
    
    QueryInterface: function(aIID) {
        if(aIID.equals(Components.interfaces.nsIWebProgressListener) ||
           aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
           aIID.equals(Components.interfaces.nsISupports)) {           
            return this;
        }
        throw Components.results.NS_NOINTERFACE;
    },
    
    /**
     * onStateChange may be fired more than one time per page so we shouldn't use
     * it to monitor page loading. More over there is no state for document loading.
     * BRO_STATE_STOP is fired only when page is fully loaded.
     * So we decided to rather use the onProgressChange event.
     * 
     * Anyway we still use this event to initialize the start loading flag.
     */
    onStateChange: function(aProgress, aRequest, aFlag, aStatus) {
        
    	if(BRO_toolbar.isRecording && (aFlag & BRO_STATE_START)) {
    		BRO_browserManager.getSelectedBrowser().BRO_START = true;
    		BRO_browserManager.getSelectedBrowser().BRO_IS_LOADED = false;
    	}
    },
    
    /**
     * This event is simple.
     * If the page document is still loading, then we do nothing.
     * If the page is already loaded, then we notify the server.
     * 
     * URL and accessing method are retrieved before the page is loaded.
     * There are two advantages:
     * - We can perform early actions, before the user see the next page.
     * - We are not influenced by scripts redirecting the page.
     */
    onLocationChange: function(aProgress, aRequest, aURI) {     
    	var url = BRO_browserManager.getSelectedBrowserUrl();
    	BRO_toolbar.onLocationChange(url);
    	
    	if(BRO_toolbar.isRecording) {
            var selectedBrowser = BRO_browserManager.getSelectedBrowser();
            
            // We take care that BRO_IS_LOADED flag is valid.
            if(!selectedBrowser.BRO_IS_LOADED 
               && (!aProgress.isLoadingDocument 
                   || BRO_actionListener.isLoadedFromBFCache())) {
               selectedBrowser.BRO_IS_LOADED = true;
            }

            // If the page is already loaded then we can notify the server
            if(BRO_model.isValidUrl(url) 
               && selectedBrowser.BRO_IS_LOADED) {

            	// If we can't create a valid action.
            	if(!BRO_actionListener.validateAction()){
            		BRO_actionListener.clear();
            		return;
            	}
            	
                var method = BRO_actionListener.lastAction;
                var title = BRO_tools.removeFirefoxNameFromTitle(aProgress.DOMWindow.document.title);
                var browserID = BRO_browserManager.getSelectedBrowserID();
                var time = BRO_tools.getTime();
            	
            	BRO_log.log('Location changed');
            	if(!BRO_toolbar.isCurrentTreeFull()) {
            		BRO_model.add(url,title,method,browserID,time);
            		if(method != BRO_ButtonsHandler.BRO_METHOD_TAB_CREATED) {
            			BRO_toolbar.addUrlRecorded(url);
            		}
            		BRO_actionListener.clear();
            	}
            } else {
            	BRO_browserManager.getSelectedBrowser().BRO_IS_LOADED = false;
            }
        }
    },

    /**
     * This event is fired each time a progression is detected.
     * So at each step, if we haven't marked the current loading proccess has 'LOADED'
     * then we wait for it to load.
     * 
     * Currently we wait for a valid URL and a valid title.
     * Title is in fact the browser title, which is the <title> value for HTML pages
     * or a name generated by the browser for other types of content.
     */
    onProgressChange: function(aProgress , aRequest , curSelfProgress , maxSelfProgress , curTotalProgress , maxTotalProgress) {
    	var selectedBrowser = BRO_browserManager.getSelectedBrowser();
    	
        if(BRO_toolbar.isRecording                // Broceliand is recording  
           && !selectedBrowser.BRO_IS_LOADED      // The page is not enaugh loaded
           && selectedBrowser.contentDocument     // There is an HTML document
           ) {
           	
            var title = BRO_tools.removeFirefoxNameFromTitle(selectedBrowser.contentDocument.title);
            var url = BRO_browserManager.getSelectedBrowserUrl();        
           	
            // The new page is being downloaded in background but visual page has not changed yet.
            if(selectedBrowser.BRO_START && url == BRO_actionListener.lastActionBrowserURL) {
                return;
            }else{
                selectedBrowser.BRO_START = false;
            }
            
            // If the page title is downloaded then we are ready to save it
            if(title != null && BRO_model.isValidUrl(url)) {
                
            	selectedBrowser.BRO_IS_LOADED = true;
            	
                // If we can't create a valid action.
                if(!BRO_actionListener.validateAction()){
                    BRO_actionListener.clear();
                    return;
                }
                
                var method = BRO_actionListener.lastAction;
                var browserID = BRO_browserManager.getSelectedBrowserID();
                var time = BRO_tools.getTime();
                
                BRO_log.log('Loaded');
                if (!BRO_toolbar.isCurrentTreeFull()) {
	                BRO_model.add(url, title, method, browserID, time);
	                BRO_toolbar.addUrlRecorded(url);
	                BRO_actionListener.clear();
                }
                BRO_actionListener.lastPageLoadedTime = BRO_tools.getTime();
            }
        }
    },
    
    // For definitions of the remaining functions see XULPlanet.com
    onStatusChange: function() {return 0;},
    onSecurityChange: function() {return 0;},
    onLinkIconAvailable: function() {return 0;}
    
}

/**
* Extension Manager Listener
*/
var BRO_extensionManagerListener = {
	
	_observerService:null,
		
	init:function() {
		BRO_extensionManagerListener._observerService = Components.classes["@mozilla.org/observer-service;1"]
		                                                          .getService(Components.interfaces.nsIObserverService);	
		BRO_extensionManagerListener.registerListener();
	},
	
	registerListener:function() {		
		if(BRO_extensionManagerListener._observerService) {			
			BRO_extensionManagerListener._observerService.addObserver(BRO_extensionManagerListener, "em-action-requested", false);
		}		
	},
	
	unregisterListener: function() {
		if(BRO_extensionManagerListener._observerService) {			
			BRO_extensionManagerListener._observerService.removeObserver(BRO_extensionManagerListener, "em-action-requested", false);	
		}
	},
	
	observe: function (subject, topic, state) {
		if(Components && Components.interfaces && Components.interfaces.nsIUpdateItem) {
			var extension = subject.QueryInterface(Components.interfaces.nsIUpdateItem);
		}
		
		if(extension && extension.id == BRO_ADDON_ID) {
		    if(topic == "em-action-requested") {
		    	if(state == "item-uninstalled") {
		    		BRO_toolbar.onUninstall();
		    	}
		    	else if(state == "item-installed") {}
		    	else if(state == "item-upgraded") {}
		    }
		}
	}
}

/**
 * Download Listener
 * 
 * If the user download a file after cli cking an URL, we save the file URL.
 */
var BRO_downloadListener = {
	
	_observerService:null,
		
	init:function() {
		BRO_downloadListener._observerService = Components.classes["@mozilla.org/observer-service;1"]
                                               	.getService(Components.interfaces.nsIObserverService);
	},
	
	registerListener:function() {
		if(BRO_downloadListener._observerService) {			
			BRO_downloadListener._observerService.addObserver(BRO_downloadListener, "dl-done", false);
		}		
	},
	
	unregisterListener: function() {
		if(BRO_downloadListener._observerService) {			
			BRO_downloadListener._observerService.removeObserver(BRO_downloadListener, "dl-done", false);	
		}
	},
	
	observe: function (subject, topic, state) {
		if(!Components || !Components.interfaces.nsIDownload) return;
	    var oDownload = subject.QueryInterface(Components.interfaces.nsIDownload);
	    // File object
	    var oFile = null;
	    try{
	      oFile = oDownload.targetFile;  // New firefox 0.9+
	    } catch (e){
	      oFile = oDownload.target;      // Old firefox 0.8
	    }
	    if(topic == "dl-done") {
	    	if(oDownload.source) var url = oDownload.source.spec;
	    	var title = oDownload.displayName;
	    	
            if(title != null && BRO_model.isValidUrl(url)) {
                
                var method = BRO_actionListener.lastAction;
                var browserID = BRO_browserManager.getSelectedBrowserID();
                var time = BRO_tools.getTime();
                
                BRO_log.log('file downloaded');
                if(!BRO_toolbar.isCurrentTreeFull()) {
                	BRO_model.add(url, title, method, browserID, time);
                	if(method != BRO_ButtonsHandler.BRO_METHOD_TAB_CREATED) {
                		BRO_toolbar.addUrlRecorded(url);
                	}
                	BRO_actionListener.clear();
                }
            }
	    }
	}
}

/**
 * We listen to various extensions and components 
 * such as the tabs or the search text box.
 */
var BRO_actionListener = {
	
	lastAction:BRO_METHOD_UNKNOWN,
	
	/**
	 * When the event took place
	 * @type int
	 */
	lastActionTime:null,
    /**
     * Specific for clicks.
     * @type int
     */	
	lastClickActionTime:null,
	/**
	 * Specific for detecting JS page reloads
	 * @type int
	 */
	lastLocationChangeTime:null,
	
	/**
	 * @type string
	 */
	lastActionBrowserID:null,
	
    /**
     * @type string
     */
    lastActionBrowserURL:null,
    
    /**
     * @type browser
     */
    lastActionBrowser:null,    
	
	init: function() {
		BRO_actionListener.setLastAction(BRO_METHOD_UNKNOWN);
	},
	
	/**
	 * Must be used for updating the lastAction value.
	 * @param int lastAction id
	 */
	setLastAction:function(action) {
		BRO_actionListener.lastAction = action;
		var time = BRO_tools.getTime();
		BRO_actionListener.lastActionTime = time;
		// We keep clicks time longer in order to detect unpredicted actions such as javascript.
		if(action == BRO_METHOD_LINK_CLICKED 
		   || action == BRO_METHOD_BODY_CLICKED) {
			BRO_actionListener.lastClickActionTime = time;
		}
	},
	
	/**
	 * Clear the last action.
	 * Should be called after each server notification.
	 */
	clear:function() {
		BRO_actionListener.setLastAction(BRO_METHOD_UNKNOWN);
		
		// We keep track of the last action information.
        if(BRO_browserManager.getSelectedBrowser()) {
            BRO_actionListener.lastActionBrowserID = BRO_browserManager.getSelectedBrowserID();
            BRO_actionListener.lastActionBrowserURL = BRO_browserManager.getSelectedBrowserUrl();
            BRO_actionListener.lastActionBrowser = BRO_browserManager.getSelectedBrowser();
        }		
	},
	
	/**
	 * Check the validity of the method before sending to the server.
	 * @return {Boolean}
	 */
	isServerMethod:function(){
		return (BRO_actionListener.lastAction >= 0 && BRO_actionListener.lastAction <=7 )?true:false;
	},
	
	/**
	 * Validate the current action before sending to the server.
	 */
	validateAction:function() {
		BRO_actionListener.checkJavascriptAction();
		
		if(BRO_actionListener.isRedirection()) {
		   	//BRO_actionListener.setLastAction(BRO_METHOD_IGNORE);
			//return false;
			BRO_actionListener.setLastAction(BRO_METHOD_LINK_CLICKED);
		}
		else if(!BRO_actionListener.isServerMethod()) {
			BRO_actionListener.setLastAction(BRO_METHOD_UNKNOWN);
		}
		return true;
	},
	
	/**
	 * Update the current action method if we think javascript has been used.
	 * We take a look at:
	 * - Javascript reloading the page
	 * - Javascript links and buttons.
	 */
	checkJavascriptAction:function() {
		
		if(BRO_actionListener.isJavascriptLinkAction()){
            BRO_actionListener.setLastAction(BRO_METHOD_LINK_CLICKED);
		}
		
	},
	
	/**
	 * Is the current action a javascript link or button action ?
	 * @return {Boolean}
	 */
	isJavascriptLinkAction:function() {
		var time = BRO_tools.getTime();
		// Several tests around 400 ms. Some at 1264 ms. But Facebook log in around 7000 ms !
		//BRO_log.log('Javascript time: '+parseInt(time - BRO_actionListener.lastClickActionTime));
        if(BRO_actionListener.lastAction == BRO_METHOD_BODY_CLICKED
           && parseInt(time - BRO_actionListener.lastClickActionTime) < BRO_JS_TIME_TO_CHANGE_LOCATION
           && (BRO_browserManager.getSelectedBrowserID() == BRO_actionListener.lastActionBrowserID
               || BRO_browserManager.getSelectedBrowser().BRO_PARENT_UID == BRO_actionListener.lastActionBrowserID)) {
            
            BRO_log.log('isJavascriptLinkAction');
            return true;
        }
	},
	
    /**
     * Was the last action a redirection ?
     * @param string currentUrl
     * @return {Boolean}
     */
    isRedirection:function() {
        var time = BRO_tools.getTime();
        /*
        BRO_log.log(BRO_actionListener.lastAction+' == '+BRO_METHOD_UNKNOWN);
        BRO_log.log(BRO_browserManager.getSelectedBrowserID()+' == '+BRO_actionListener.lastActionBrowserID);
        BRO_log.log(parseInt(time - BRO_actionListener.lastPageLoadedTime)+' < '+BRO_TIME_TO_REDIRECT);
        */
        if(BRO_actionListener.lastAction == BRO_METHOD_UNKNOWN
           && BRO_browserManager.getSelectedBrowserID() == BRO_actionListener.lastActionBrowserID
           && BRO_browserManager.getSelectedBrowser() == BRO_actionListener.lastActionBrowser
           && parseInt(time - BRO_actionListener.lastPageLoadedTime) < BRO_TIME_TO_REDIRECT) {
            
           	BRO_log.log('isRedirection');
            return true;
        }else{
            return false;
        }    	
    },
    
    isLoadedFromBFCache:function(){
    	return (BRO_actionListener.lastAction == BRO_METHOD_BACK_BUTTON
    	        || BRO_actionListener.lastAction == BRO_METHOD_FORWARD_BUTTON)?true:false;
    },
	
    /**
     * We listen tab selections.
     */
    tabSelected: function(event) {       
        //BRO_log.log("Tab selected !"); 
        BRO_actionListener.setLastAction(BRO_METHOD_TAB_SELECTED);
    },
    
    /**
     * When a tab is created we have to asign a unique ID.
     * @param event
     */
    tabAdded: function(event) {
    	// We init the new tab
    	BRO_browserManager.initBrowser(event.target.linkedBrowser, BRO_browserManager.getSelectedBrowser());
    	
    	// If the tab is created from the current page, then we notice the server
    	var time = BRO_tools.getTime();
        // Several tests around 20 ms. Higher test at 889.
        //BRO_log.log('Tab added in :'+time - BRO_actionListener.lastClickActionTime+' ms');    	    	
    	if(BRO_toolbar.isRecording 
    	   && parseInt(time - BRO_actionListener.lastClickActionTime) < BRO_TIME_TO_CREATE_TAB
    	   && (BRO_actionListener.lastAction == BRO_METHOD_LINK_CLICKED
    	      || BRO_actionListener.isJavascriptLinkAction())) {	
    		
    	   // We use the current URL to show the relationship with this new tab.
    	   var url = BRO_browserManager.getSelectedBrowser().currentURI.spec;
    	   var title = '';
    	   var method = BRO_METHOD_TAB_CREATED;
    	   var browserID = event.target.linkedBrowser.BRO_UID;
    	   BRO_model.add(url,title,method,browserID,time);
    	}
    	
    	//BRO_log.log('new browser: '+event.target.linkedBrowser.BRO_UID);
    },  
    
    /**
     * Listen to clicks.
     * - If the target is an anchor there are no doubts.
     * - We also register clicks over the page for testing javascript actions.
     * 
     * @param event
     */
    onClick: function(event) {    	
    	if (!event.target) return;
    	var target = event.originalTarget;
    	while (target.nodeName != 'A' && target.nodeName != 'BODY' && target.parentNode){
    		target = target.parentNode;
    	}
    	if (target.nodeName == 'A'){
    		//0: Normal button. Left button for non Macintosh mice.
    		//1: Middle button (on the wheel).
    		if(event.button == 0 || event.button == 1){
    			//BRO_log.log("Link clicked !");
    			BRO_actionListener.setLastAction(BRO_METHOD_LINK_CLICKED);
    		}
    		//2: Display context menu
    		else if(event.button == 2){
    			//...do nothing
    		}
    		else{
    			//...do nothing but some mice may have navigation buttons.
    		}    		
    	}else if(target.nodeName == 'BODY') {
    		if(event.button == 0) {
    			BRO_actionListener.setLastAction(BRO_METHOD_BODY_CLICKED);
    		}
    	}
    },
    
    /**
     * Context menu opening in a new tab.
     */
    openLinkInTab:function() {
    	//BRO_log.log("Open in a new tab"); 
    	BRO_actionListener.setLastAction(BRO_METHOD_LINK_CLICKED);
    },

    /**
     * Context menu opening in a new window.
     * The new window created will be able to retreive the opener ID.
     */
    openLinkInWindow:function() {
        BRO_log.log("Open in a new window"); 
    	BRO_windowManager.setWindowOpenerID(BRO_browserManager.getSelectedBrowserID());
    },
    
    /**
     * Listen form submits
     * @param event
     */
    onSubmit: function(event){
        //BRO_log.log("Form submited !");
        BRO_actionListener.setLastAction(BRO_METHOD_FORM_SUBMITED);
    },
    
    /**
     * Listen forward button
     * @param event
     */
    onClickForward: function(event){
    	//BRO_log.log("Form submited !");
        BRO_actionListener.setLastAction(BRO_METHOD_FORWARD_BUTTON);
    },
    
    /**
     * Listen back button
     * @param event
     */
    onClickBack: function(event){
        //BRO_log.log("Form submited !");
        BRO_actionListener.setLastAction(BRO_METHOD_BACK_BUTTON);
    },
    
    /**
     * Listen refresh button.
     * @param event
     */
    onClickReload: function(event){
    	// By setting the action to unknown we won't record it
    	BRO_actionListener.setLastAction(BRO_METHOD_IGNORE);
    },
    
    /**
     * We listen the Home Button
     */
    onClickHomeButton: function(event) {
    	//BRO_log.log("Homme button clicked.");
    	BRO_actionListener.setLastAction(BRO_METHOD_CLICK_HOME);
    },
    
    /**
     * We listen the search Button
     */
    onClickSearchButton: function(event) {
        //BRO_log.log("Search button clicked.");
        BRO_actionListener.setLastAction(BRO_METHOD_SEARCH);
    },
    
    /**
     * We listen the search text field
     */
    onPressKeySearch: function(event) {
        //BRO_log.log("Search request sent by key.");
        BRO_actionListener.setLastAction(BRO_METHOD_SEARCH);
    },
    
    /**
     * We listen to the URL text field
     */
    onPressUrlbar:function(event) {
	   //BRO_log.log("URL bar pressed");
	   BRO_actionListener.setLastAction(BRO_METHOD_TYPE_URL);
    },
    
    onClickBookmark:function(event) {
    	//BRO_log.log("Bookmark clicked");
    	BRO_actionListener.setLastAction(BRO_METHOD_CLICK_BOOKMARK);
    },
    
    openContextMenu:function(event) {
    	var menuItemId = event.target.id;
    	if(menuItemId == 'context-openlinkintab') {
    		BRO_actionListener.openLinkInTab()
    	}else if(menuItemId == 'context-openlink'){
    		BRO_actionListener.openLinkInWindow();
    	}
    }
}

/**
 * Manage all types of listeners
 * BRO_navListener for window content
 * BRO_actionListener for other extensions and components
 */
var BRO_listenerHandler = {
	
    isRegistered:false,	
	
	init: function() {		
        // Listeners are started / stopped when the user clicks on the record button
        window.addEventListener("unload",BRO_listenerHandler.unloadHandler, false);
        window.addEventListener("load", BRO_listenerHandler.loadHandler, false);
        window.addEventListener("focus", BRO_windowManager.synchronize, false);        
	},
	
	loadHandler: function(){
        if(gBrowser) {
        	gBrowser.addProgressListener(BRO_navListener , BRO_NOTIFY_STATE_DOCUMENT);
        }
	},
	
	unloadHandler: function(){
		BRO_listenerHandler.unregisterListeners();
		if(gBrowser) {
			gBrowser.removeProgressListener(BRO_navListener);
		}
		var windows = BRO_windowManager.countWindows();
        if(windows==0) {
            BRO_toolbar.onQuit();
        }
	},
	
	registerListeners: function() { 
        if(BRO_listenerHandler.isRegistered) return;
        else BRO_listenerHandler.isRegistered = true;
        
        if(BRO_toolbar.recordingMode == RECORING_MODE_CONTINUOUS) {
        	BRO_listenerHandler.registerListenersForContinuousMode();
        }
        else{
        	BRO_listenerHandler.registerListenersForOneByOneMode();
        }
	},
	
    unregisterListeners: function() {
        if(!BRO_listenerHandler.isRegistered) return;
        else BRO_listenerHandler.isRegistered = false;
        
        if(BRO_toolbar.recordingMode == RECORING_MODE_CONTINUOUS) {
        	BRO_listenerHandler.unregisterListenersForContinuousMode();
        }
        else{
        	BRO_listenerHandler.unregisterListenersForOneByOneMode();
        }
    },	
	
	registerListenersForOneByOneMode: function() {
		// no special listeners for one by one mode
	},
    
    unregisterListenersForOneByOneMode:function() {
		// no special listeners for one by one mode
    },	
	
	registerListenersForContinuousMode: function() {    	
        BRO_downloadListener.registerListener();
        
    	if(document.getElementById('personal-bookmarks')) {
    		document.getElementById('personal-bookmarks').addEventListener('click', BRO_actionListener.onClickBookmark, false);
    	}else{
    		BRO_log.log("Listener Error: Can't find the personal toolbar");
    	}
    	
    	if(document.getElementById('urlbar')) {
    	   document.getElementById('urlbar').addEventListener('input',BRO_actionListener.onPressUrlbar, false);
    	   /**
    	    * @todo This eventerListener doesn't work but should
    	    */
    	   //document.getElementById('urlbar').addEventListener('textentered',BRO_actionListener.onPressUrlbar, true);
    	}else{
    	   BRO_log.log("Listener Error: Can't find the URL bar");
    	}
    	
    	if(document.getElementById('searchbar') && document.getAnonymousElementByAttribute(document.getElementById('searchbar'), "anonid", "search-go-button")) {
    	   document.getAnonymousElementByAttribute(document.getElementById('searchbar'), "anonid", "search-go-button").addEventListener('click', BRO_actionListener.onClickSearchButton, false);
    	   document.getElementById('searchbar').addEventListener('input',BRO_actionListener.onPressKeySearch, false);
    	}else{
    		BRO_log.log("Listener Error: Can't find the search button");
    	}
    	
    	if(document.getElementById('home-button')) {
    	   document.getElementById('home-button').addEventListener('click', BRO_actionListener.onClickHomeButton, false);
    	}else{
    		BRO_log.log("Listener Error: Can't find home button");
    	}
    	
    	if(document.getElementById("reload-button")) {
    	   document.getElementById("reload-button").addEventListener('click', BRO_actionListener.onClickReload, true);
    	}else{
    		BRO_log.log("Listener Error: Can't find reload button");
    	}
    	
    	if(document.getElementById("forward-button") && document.getElementById("back-button")) {
    	   document.getElementById("forward-button").addEventListener('click', BRO_actionListener.onClickForward, true);
    	   document.getElementById("back-button").addEventListener('click', BRO_actionListener.onClickBack, true);
    	}else {
    		BRO_log.log("Listener Error: Can't find back / forward buttons");
    	}
    	
    	if(document.getElementById("contentAreaContextMenu")) {
    	   document.getElementById("contentAreaContextMenu").addEventListener('command', BRO_actionListener.openContextMenu, true);
    	}else {
    		BRO_log.log("Listener Error: Can't find the context menu");
    	}
    	
    	if(document.getElementById("context-back") && document.getElementById("context-forward")){
            document.getElementById("context-forward").addEventListener('click', BRO_actionListener.onClickForward, true);
            document.getElementById("context-back").addEventListener('click', BRO_actionListener.onClickBack, true);    		
    	}else{
            BRO_log.log("Listener Error: Can't find back / forward buttons in the context menu");
    	}
    	
    	if(document.getElementById("context-reload")){
           document.getElementById("context-reload").addEventListener('click', BRO_actionListener.onClickReload, true);
        }else{
            BRO_log.log("Listener Error: Can't find reload item in the context menu");
        }
    	
    	if(gBrowser) {
        	gBrowser.addEventListener("submit", BRO_actionListener.onSubmit, true);
        	gBrowser.addEventListener("click", BRO_actionListener.onClick, true);  

        	if(gBrowser.mPanelContainer) {
        	   gBrowser.mPanelContainer.addEventListener("select", BRO_actionListener.tabSelected, false);
        	}else{
        		BRO_log.log("Listener Error: Can't listen to tab selection");
        	}
        	if(gBrowser.tabContainer) {
                gBrowser.tabContainer.addEventListener("TabOpen", BRO_actionListener.tabAdded, false);
        	}else {
        		BRO_log.log("Listener Error: Can't listen to tab creation");
        	}
    	}else {
    		BRO_log.log("Listener Error: gBrowser is null");
    	}
    },
    
    unregisterListenersForContinuousMode:function() {    	
        BRO_downloadListener.unregisterListener();
        
    	if(document.getElementById('personal-bookmarks')) {
            document.getElementById('personal-bookmarks').removeEventListener('click', BRO_actionListener.onClickBookmark, false);
        }
        
    	if(document.getElementById('urlbar')) {
           document.getElementById('urlbar').removeEventListener('input',BRO_actionListener.onPressUrlbar, false);
    	}
    	
        if(document.getElementById('searchbar') && document.getAnonymousElementByAttribute(document.getElementById('searchbar'), "anonid", "search-go-button")) {
    	   document.getAnonymousElementByAttribute(document.getElementById('searchbar'), "anonid", "search-go-button").removeEventListener('click', BRO_actionListener.onClickSearchButton, false);
           document.getElementById('searchbar').removeEventListener('input',BRO_actionListener.onPressKeySearch, false);
        }
        
        if(document.getElementById('home-button')) {
    	   document.getElementById('home-button').removeEventListener('click', BRO_actionListener.onClickHomeButton, false);
        }
        
    	if(document.getElementById("reload-button")) {
    	   document.getElementById("reload-button").removeEventListener('click', BRO_actionListener.onClickReload, true);
    	}
    	
        if(document.getElementById("context-reload")){
           document.getElementById("context-reload").removeEventListener('click', BRO_actionListener.onClickReload, true);
        }   	
    	
    	if(document.getElementById("forward-button") && document.getElementById("back-button")) {
    	   document.getElementById("forward-button").removeEventListener('click', BRO_actionListener.onClickForward, true);
           document.getElementById("back-button").removeEventListener('click', BRO_actionListener.onClickBack, true);
    	}
    	
        if(document.getElementById("context-back") && document.getElementById("context-forward")){
            document.getElementById("context-forward").removeEventListener('click', BRO_actionListener.onClickForward, true);
            document.getElementById("context-back").removeEventListener('click', BRO_actionListener.onClickBack, true);            
        }  	
    	
    	if(document.getElementById("contentAreaContextMenu")) {
    		document.getElementById("contentAreaContextMenu").removeEventListener('command', BRO_actionListener.openContextMenu, true);
    	}
    	
    	if(gBrowser){
        	gBrowser.removeEventListener("submit", BRO_actionListener.onSubmit, true);
        	gBrowser.removeEventListener("click", BRO_actionListener.onClick, true);
            if(gBrowser.mPanelContainer){
                gBrowser.mPanelContainer.removeEventListener("select", BRO_actionListener.tabSelected, false);
            }
            if(gBrowser.tabContainer) {
                gBrowser.tabContainer.removeEventListener("TabOpen", BRO_actionListener.tabAdded, false);
            }    
    	}
    }
}